/* vector.cpp
 *
 * Pieter Eendebak ( pte@ddsw.nl )
 *
 */

#include <math.h>
#include "vector.h"

Vector::Vector () 
	: data(0)
{
	create(3);
	set(0,0,0);
}

Vector::Vector (int l)
	: data(0)
{
	create(l);
}

Vector::Vector ( double a, double b, double c )
	: data(0)
{
	create(3);
	set(a,b,c);
}

void Vector::create()
{
	if (data!=0) delete [] data;
	data = new double [length];
}

void Vector::create (int l)
{
	if (l<1) l=1;

	length=l;
	create();
}

void Vector::set ( Vector v )
{
	create( v.size() );
	for(int i=0;i<length;i++)
		data[i]=v[i];
}

void Vector::operator= (Vector v)
{
	set( v );
}

void Vector::operator= (Vector *v)
{
	set( *v );
}

Vector::~Vector()
{
	// delete [] data;
}

char * Vector::toString()
{
	QString tmp, tmp2;
	tmp.sprintf("vector : ");
	for(int i=0;i<length;i++)
	{
		tmp2.sprintf("%f ", data[i] );
		tmp.append( tmp2.data() );
	}
	return tmp.data();
}

int Vector::size()
{
	return length;
}

void Vector::set(double a, double b, double c)
{
	create(3);

	data[0]=a;
	data[1]=b;
	data[2]=c;
}

bool Vector::equals ( Vector v )
{
	if (length!=v.length) return false;
	else
	{
		for(int i=0;i<length;i++)
		{
			if ( data[i]!=v.data[i] ) return false;
		}
		return false;
	}
}

double Vector::get (int index)
{
	if ( index<0 || index>= length ) return 0;
	else return data[index];
}

double& Vector::x()
{
	return data[0];
}

double& Vector::y()
{
	return data[1];
}

double& Vector::z()
{
	return data[2];
}

double& Vector::operator[] ( int i )
{
	if (i<0 || i>=length) return data[0];
	else return data[i];
}

bool Vector::isZero()
{
	if (norm()==0) return true;
	else return false; 
}

double Vector::scalProd(Vector v)
{
	double ret=0;

	if( size()==v.size() )
	{
		for(int i=0;i<length;i++)
		{
			ret+=data[i]*v[i];
		}
	}

	return ret;
}

double Vector::operator* (Vector v)
{
	return scalProd(v);
}

double Vector::norm()
{
	double tmp=0;

	for(int i=0;i<length;i++)
	{
		tmp+=data[i]*data[i];
	}
	return sqrt( tmp );
}

void Vector::Normalize()
{
	double n = norm();

	for(int i=0;i<length;i++)
	{
		data[i]=data[i]/n;
	}
}

double Vector::cosAngle(Vector v)
{
	if ( v.isZero() || isZero() ) return 0;
	else
	{
		return ( scalProd(v)/( norm()*v.norm() ) );
	}
}

void Vector::scalMult( double s)
{
	for(int i=0;i<length;i++)
	{
		data[i]*=s;
	}
}

Vector Vector::operator* ( double s )
{
	Vector ret = *this;
	ret.scalMult( s );
	return ret;
}

void Vector::operator*= ( double s )
{
	scalMult( s );
}

void Vector::addVec( Vector v )
{
	int max = size()<v.size()?size():v.size();

	for(int i=0;i<max;i++)
	{
		data[i]+=v.get(i);
	}
}

Vector Vector::operator+ ( Vector v )
{
	Vector ret = v;
	ret.addVec(*this);
	return ret;
}

void Vector::operator+= ( Vector v )
{
	addVec(v);
}

void Vector::subVec( Vector v )
{
	int max = size()<v.size()?size():v.size();
	for(int i=0;i<max;i++)
	{
		data[i]-=v.get(i);
	}
}

Vector Vector::operator- ( Vector v )
{
	Vector ret = *this;
	ret.subVec( v );
	return ret;
}

void Vector::operator-= ( Vector v )
{
	subVec(v);
}

void Vector::copyVec( Vector v )
{
	set( v );
}

void Vector::copyVec( Vector *v )
{
	copyVec(*v);
}

void Vector::vectorProd( Vector v, Vector w )
{
	if ( v.size()==3 && w.size()==3 )
	{
		data[0]= v.get(1)*w.get(2) - v.get(2)*w.get(1);
		data[1]= -v.get(0)*w.get(2) + v.get(2)*w.get(0);
		data[2]= v.get(0)*w.get(1) - v.get(1)*w.get(0);
	}
}
